# -*- coding: utf-8 -*-
from Acquisition import aq_base
from collective.contentrules.mustread import config
from collective.contentrules.mustread.actions.mail import MustReadAction
from collective.contentrules.mustread.actions.mail import MustReadAddForm
from collective.contentrules.mustread.actions.mail import MustReadEditForm
from collective.contentrules.mustread.actions.mail import MustReadReminderExecutor  # noqa
from collective.contentrules.mustread.actions.mail import MustReadSendConfirmationExecutor  # noqa
from collective.contentrules.mustread.event import ReadReminderEvent
from collective.contentrules.mustread.testing import COLLECTIVE_CONTENTRULES_MUSTREAD_INTEGRATION_TESTING  # noqa
from collective.mustread.testing import tempDb
from collective.mustread.tracker import Tracker
from datetime import datetime
from datetime import time
from datetime import timedelta
from email import message_from_string
from plone import api
from plone.app.contentrules.rule import Rule
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleAction
from Products.CMFPlone.tests.utils import MockMailHost
from Products.MailHost.interfaces import IMailHost
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.interface import implements

import re
import unittest


class DummyEvent(object):
    """Event representing any content-rule related event,
    not mustread specific"""
    implements(IObjectEvent)

    def __init__(self, object):
        self.object = object


class TestMustReadContentRule(unittest.TestCase):
    """Test the mustread content rule
    (test based on collective.contentrules.mailtorole)
    """

    layer = COLLECTIVE_CONTENTRULES_MUSTREAD_INTEGRATION_TESTING

    def setUp(self):
        # setup mustread db for testing
        self.db = tempDb()
        self.tracker = Tracker()

        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.installProduct('collective.contentrules.mustread')

        # replace mailhost
        # this needs a new version of plone.app.testing:
        # see https://github.com/plone/plone.app.testing/pull/40
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost('MailHost')
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.folder = api.content.create(self.portal, 'Folder', title='Folder')
        self.page1 = api.content.create(
            self.folder, 'Document', title='Page 1')

        user1 = api.user.create('user1@plone.org', 'user1',
                                properties={'fullname': u'User 1 Fullname'})
        user2 = api.user.create('user2@plone.org', 'user2',
                                properties={'fullname': u'John Döe'})
        group1 = api.group.create('group1')
        api.group.add_user(group=group1, user=user1)
        group2 = api.group.create('group2')
        api.group.add_user(group=group2, user=user2)
        api.group.create(
            'mega', u'Mega Group',
            description=u'Group containing group1 and group2',
            groups=['group1', 'group2'])

    def tearDown(self):
        # restore mailhost
        self.portal.MailHost = self.portal._original_MailHost
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(aq_base(self.portal._original_MailHost),
                           provided=IMailHost)

    def test_registered(self):
        element = getUtility(IRuleAction, name='plone.actions.MustRead')
        self.assertEqual('plone.actions.MustRead', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(IObjectEvent, element.event)

    def test_addview(self):
        element = getUtility(IRuleAction, name='plone.actions.MustRead')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+action')
        addview = getMultiAdapter((adding, self.portal.REQUEST),
                                  name=element.addview)
        self.assertTrue(isinstance(addview, MustReadAddForm))

        addview.createAndAdd(data=dict(
            notification_subject=config.DEFAULT_NOTIFICATION_SUBJECT,
            reminder_subject=config.DEFAULT_REMINDER_SUBJECT,
            role=u'Owner',
            notification_message=config.DEFAULT_NOTIFICATION_TEXT,
            reminder_message=config.DEFAULT_REMINDER_TEXT,
            acquired=config.DEFAULT_ACQUIRED_ROLES,
            deadline=config.DEFAULT_DEADLINE_DELAY,
            reminder_delay=config.DEFAULT_REMINDER_DELAY,
        ))

        e = rule.actions[0]
        self.assertTrue(isinstance(e, MustReadAction))
        self.assertEqual(u'Read request', e.notification_subject)
        self.assertEqual(u'Reminder: read request', e.reminder_subject)
        self.assertTrue(u'Dear ${mustread_fullname}' in e.notification_message)
        self.assertTrue(u'please make sure to read' in e.reminder_message)
        self.assertEqual(5, e.deadline)
        self.assertEqual(2, e.reminder_delay)
        self.assertEqual(u'', e.source)
        self.assertEqual(u'Owner', e.role)
        self.assertEqual(True, e.acquired)
        self.assertEqual(False, e.global_roles)

    def test_editview(self):
        element = getUtility(IRuleAction, name='plone.actions.MustRead')
        e = MustReadAction()
        editview = getMultiAdapter((e, self.portal.REQUEST),
                                   name=element.editview)
        self.assertTrue(isinstance(editview, MustReadEditForm))

    def test_affected_users(self):
        """test computation of affected users via local roles
        (acquired and not) and global_roles
        """
        e = MustReadAction()
        e.role = 'Reader'
        e.acquired = False
        e.global_roles = False
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.page1)),
                             IExecutable)
        self.assertTrue(isinstance(ex, MustReadSendConfirmationExecutor))

        # no user has reader role
        self.assertEqual(0, len(ex.affected_users))

        # reader role acquired, but action.acquired=False
        api.user.grant_roles(username='user1', obj=self.folder,
                             roles=['Reader'])
        self.assertEqual(0, len(ex.affected_users))

        # reader role directly on object
        api.user.grant_roles(username='user2', obj=self.page1,
                             roles=['Reader'])
        self.assertEqual(ex.affected_users, ['user2'])

        # set action.acquired=False, both users should be returned
        e.acquired = True
        self.assertEqual(sorted(ex.affected_users), ['user1', 'user2'])

        # test global roles
        e.role = 'Editor'
        e.global_roles = True
        self.assertEqual(0, len(ex.affected_users))
        api.user.grant_roles('user1', roles=['Editor'])
        self.assertEqual(ex.affected_users, ['user1'])

    def test_affected_groups(self):
        """test computation of affected users via groups
        """
        e = MustReadAction()
        e.role = 'Reader'
        e.acquired = False
        e.global_roles = False
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.page1)),
                             IExecutable)

        # make group1, readers, user1 should be affected
        self.assertEqual(0, len(ex.affected_users))
        api.group.grant_roles('group1', obj=self.page1, roles=['Reader'])
        self.assertEqual(ex.affected_users, ['user1'])

        # make megagroup editors, both users should be affected
        e.role = 'Editor'
        e.acquired = True
        self.assertEqual(0, len(ex.affected_users))
        api.group.grant_roles('mega', obj=self.page1, roles=['Editor'])
        self.assertEqual(sorted(ex.affected_users), ['user1', 'user2'])

    def test_stringinterp(self):
        """test if mustread_ placeholders for plone.stringinterp work
        for the mustread-executors"""
        e = MustReadAction()
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.page1)),
                             IExecutable)

        interpolator = ex._get_interpolator('user1')

        self.assertEqual(interpolator(u'Hello ${mustread_fullname}'),
                         u'Hello User 1 Fullname')

        self.assertEqual(
            interpolator(u'url: ${mustread_url}'),
            'url: http://nohost/plone/folder/page-1/@@mark-read')

        # we need a mustread db entry for this user and page in order to
        # obtain the deadline
        self.assertEqual(interpolator(u'deadline: ${mustread_deadline}'),
                         'deadline: ')
        self.tracker.schedule_must_read(self.page1, ['user1'],
                                        datetime(2017, 4, 12, 13, 00))
        interpolator = ex._get_interpolator('user1')
        self.assertEqual(interpolator(u'deadline: ${mustread_deadline}'),
                         'deadline: Apr 12, 2017')

    def test_email(self):
        """test sending of emails

        * no source email given, use portal settings
        * string interpolation
        """
        e = MustReadAction()
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.page1)),
                             IExecutable)

        self.assertRaises(ValueError, ex._send_mail,
                          users=['user1'], subject='Subject', text='Text')

        # if we provide a site mail address this won't fail anymore
        sm = getSiteManager(context=self.portal)
        sm.manage_changeProperties({'email_from_address': 'admin@site.com',
                                    'email_from_name': 'Website Adminstratör'})
        ex._send_mail(['user1'],
                      u'Notification for ${mustread_fullname}',
                      u'Please read ${mustread_url}')

        messages = self.portal.MailHost.messages
        msg = message_from_string(messages[0])
        # from address is the one we defined on the portal
        self.assertEqual(
            msg['From'],
            '=?utf-8?q?Website_Adminstrat=C3=B6r?= <admin@site.com>')
        self.assertEqual(msg['Subject'],
                         '=?utf-8?q?Notification_for_User_1_Fullname?=')

    def test_confirmation_executor(self):
        """test if mustread entries are created and
        the correct users receive an email
        """
        e = MustReadAction()
        e.role = 'Reader'
        e.acquired = True
        e.global_roles = False
        e.deadline = 10
        e.notification_subject = u'Please read ${title}'
        e.notification_message = (
            u'Dear ${mustread_fullname}\nPlease read ${mustread_url} '
            u'by ${mustread_deadline} the latest.')
        e.source = '"Administrator" <admin@portal.com>'
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.page1)),
                             IExecutable)

        api.user.grant_roles('user1', obj=self.page1, roles=['Reader'])
        api.user.grant_roles('user2', obj=self.folder, roles=['Reader'])

        # no mustreads sheduled for our document
        self.assertEqual(len(self.tracker.who_did_not_read(self.page1)), 0)

        # execute
        ex()

        # 2 mustreads have been scheduled
        self.assertEqual(sorted(
            self.tracker.who_did_not_read(self.page1).keys()),
            ['user1', 'user2'])
        # the deadline is 10 days from today on
        deadline = datetime.utcnow() + timedelta(10)
        self.assertEqual(
            self.tracker.who_did_not_read(self.page1)['user1'].date(),
            deadline.date())

        # the user that invoked the action gets feedback in a status message
        messages = IStatusMessage(self.folder.REQUEST).show()  # noqa
        self.assertEqualEllipsis(
            messages[0].message,
            u'Read confirmation requested by ... for 2 users: user1, user2')

        # a notification has been sent to both users.
        messages = self.portal.MailHost.messages
        self.assertEqual(len(messages), 2)

        msg1 = message_from_string(messages[0])
        self.assertEqual(msg1['To'],
                         u'User 1 Fullname <user1@plone.org>')

        msg2 = message_from_string(messages[1])
        # from address is the one we defined as source
        self.assertEqual(msg2['From'],
                         'Administrator <admin@portal.com>')
        self.assertEqual(msg2['To'],
                         u'=?utf-8?q?John_D=C3=B6e?= <user2@plone.org>')
        self.assertEqual(msg2['Subject'],
                         '=?utf-8?q?Please_read_Page_1?=')

        self.assertEqualEllipsis(
            msg2.get_payload(),
            (u'Dear John D=C3=B6e\nPlease read '
             u'http://nohost/plone/folder/page-1/@@mark-read '
             u'by ... t=\nhe latest.'))

    def test_reminder(self):
        """
        - no text - no reminder
        - correct date computation
        """
        e = MustReadAction()
        e.role = 'Reader'
        e.reminder_delay = 2
        e.reminder_subject = u'Reminder, please read ${title}'
        e.reminder_message = (
            u'Dear ${mustread_fullname}\nPlease do not forget to '
            u'read ${mustread_url} by ${mustread_deadline} the latest.')
        e.source = '"Administrator" <admin@portal.com>'
        ex = getMultiAdapter((self.folder, e, ReadReminderEvent(self.page1)),
                             IExecutable)

        self.assertTrue(isinstance(ex, MustReadReminderExecutor))

        # create a mustread entry for user1 with
        # deadline 23:59:59 in 3 days
        # and user2 at 00:00:00 in 3 days
        dl1 = datetime.combine(datetime.utcnow(), time.max) + timedelta(3)
        dl2 = datetime.combine(datetime.utcnow(), time.min) + timedelta(3)
        self.tracker.schedule_must_read(self.page1, ['user1'], dl1)
        self.tracker.schedule_must_read(self.page1, ['user2'], dl2)
        # and another one at a later point for another user
        api.user.create('user3@plone.org', 'user3')
        self.tracker.schedule_must_read(self.page1, ['user3'],
                                        dl2 + timedelta(1))
        # as our reminder_delay is 2 days before the deadline and
        # the deadline is today+3 days no reminders will be triggered
        ex()

        messages = self.portal.MailHost.messages
        self.assertEqual(len(messages), 0)

        # we change the reminder_delay to 3 days, now reminders are sent out
        e.reminder_delay = 3
        ex()
        messages = self.portal.MailHost.messages
        self.assertEqual(len(messages), 2)
        msg = message_from_string(messages[0])
        self.assertEqual(msg['To'],
                         '=?utf-8?q?John_D=C3=B6e?= <user2@plone.org>')
        msg = message_from_string(messages[1])
        self.assertEqual(msg['To'],
                         'User 1 Fullname <user1@plone.org>')

        # empty the message list
        del self.portal.MailHost.messages[:]
        self.assertEqual(len(messages), 0)

        # change the reminder deadline to 4 days before the deadline.
        # now only user3 gets a reminder
        e.reminder_delay = 4
        ex()
        messages = self.portal.MailHost.messages
        self.assertEqual(len(messages), 1)
        msg = message_from_string(messages[0])
        self.assertEqual(msg['To'],
                         'user3@plone.org')

        # if no reminder message is set, no reminder is sent
        e.reminder_message = u''
        del self.portal.MailHost.messages[:]
        ex()
        self.assertEqual(len(messages), 0)

    def assertEqualEllipsis(self, first, second,
                            ellipsis_marker='...', msg=None):
        """
        Example :
            >>> self.assertEqualEllipsis('foo123bar', 'foo...bar')
        """
        if ellipsis_marker not in second:
            return first == second

        if (
            re.match(
                re.escape(second).replace(re.escape(ellipsis_marker), '(.*?)'),
                first,
                re.M | re.S
            ) is None
        ):
            self.assertMultiLineEqual(
                first,
                second,
                msg
            )



# -*- coding: utf-8 -*-
from collective.contentrules.mustread import _
from zope import schema
from zope.component.interfaces import IObjectEvent
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IMustReadSettings(Interface):
    '''registry settings
    '''

    expired_recipient = schema.List(
        title=_(u'label_expired-recipient',
                default=u'Expired recipient'),
        description=_(
            u'help_expired-recipient',
            default=(
                u'Email address(es) the expired notifications are sent to '
                u"If not given, the portal's admin address is used"
            )
        ),
        value_type=schema.TextLine(),
        required=False,
        default=[],
    )


class IMustReadEvent(IObjectEvent):
    """base class for all events that shall trigger content rules engine"""

    pass


class IReadConfirmationRequest(IMustReadEvent):
    """Zope Event to be notified when a read confirmation is requested
    for an object
    """

    pass


class IReadReminder(IMustReadEvent):
    """Zope Event to be notified when a reminder for open read
    confirmation requests shall be sent
    """

    pass


class ICollectiveContentrulesMustreadLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""

    pass


class ICanBeMarkedAsMustRead(Interface):
    """Marker for content that shows mustread actions"""

    pass



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from Acquisition import aq_base
from collective.contentrules.mustread import testing
from collective.contentrules.mustread.interfaces import IMustReadEvent
from collective.contentrules.mustread.interfaces import IMustReadSettings
from collective.contentrules.mustread.interfaces import IReadConfirmationRequest  # noqa
from collective.contentrules.mustread.interfaces import IReadReminder
from collective.mustread.testing import tempDb
from collective.mustread.tracker import Tracker
from datetime import datetime
from datetime import timedelta
from email import message_from_string
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from Products.CMFPlone.tests.utils import MockMailHost
from Products.MailHost.interfaces import IMailHost
from Products.statusmessages.interfaces import IStatusMessage
from StringIO import StringIO

import csv
import unittest
import zope.component


class EventCatcher(object):

    def __init__(self):
        self.fired = []
        self.setUp()

    def setUp(self):
        zope.component.provideHandler(self.handle_event)

    def tearDown(self):
        zope.component.getSiteManager().unregisterHandler(self.handle_event)

    def reset(self):
        self.fired = []

    @zope.component.adapter(IMustReadEvent)
    def handle_event(self, event):
        self.fired.append(event)


class TestMustReadViews(unittest.TestCase):
    """Test the mustread content rule
    (test based on collective.contentrules.mailtorole)
    """

    layer = testing.COLLECTIVE_CONTENTRULES_MUSTREAD_INTEGRATION_TESTING

    def setUp(self):
        # setup mustread db for testing
        self.db = tempDb()
        self.tracker = Tracker()

        # register handler that catches all events
        self.event_catcher = EventCatcher()

        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.installProduct('collective.contentrules.mustread')

        # replace mailhost
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost('MailHost')
        sm = zope.component.getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.folder = api.content.create(self.portal, 'Folder', title='Folder')
        self.page1 = api.content.create(
            self.folder, 'Document', title='Page 1')
        self.folder2 = api.content.create(self.portal, 'Folder',
                                          title='Folder 2')
        self.file1 = api.content.create(
            self.folder2, 'File', 'file1.txt', title='File 1')

        api.user.create('user1@plone.org', 'user1',
                        properties={'fullname': u'User One'})
        api.user.grant_roles('user1', obj=self.page1, roles=['Reader'])
        setRoles(self.portal, TEST_USER_ID, ['Member'])

    def tearDown(self):
        # restore mailhost
        self.portal.MailHost = self.portal._original_MailHost
        sm = zope.component.getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(aq_base(self.portal._original_MailHost),
                           provided=IMailHost)

        self.event_catcher.tearDown()

    def test_request_review(self):
        """test if our view triggers the correct event"""
        # ordinary members must not access our view
        self.assertRaises(Unauthorized, self.page1.restrictedTraverse,
                          '@@request-read-confirmation')

        # site admins and managers have the proper permission
        setRoles(self.portal, TEST_USER_ID, ['Site Administrator'])
        view = self.page1.restrictedTraverse('@@request-read-confirmation')
        view()
        self.assertEqual(len(self.event_catcher.fired), 1)

        event = self.event_catcher.fired[0]
        self.assertTrue(
            IReadConfirmationRequest.providedBy(event))
        self.assertEqual(event.object, self.page1)

        # for types listed in typesUseViewActionInListings
        # users get redirected to the view action
        view = self.file1.restrictedTraverse('@@request-read-confirmation')
        view()
        self.assertEqual(
            self.portal.REQUEST.response.headers['location'],
            'http://nohost/plone/folder-2/file1.txt/view')

    def test_send_reminder(self):
        # ordinary members must not access our view
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse,
                          '@@send-read-reminders')

        # site admins and managers have the proper permission
        setRoles(self.portal, TEST_USER_ID, ['Site Administrator'])

        # calling our view w/o open requests results in no events fired
        view = self.portal.restrictedTraverse('@@send-read-reminders')
        view()
        self.assertEqual(len(self.event_catcher.fired), 0)
        # a status message indicates if reminders have been sent
        messages = IStatusMessage(self.portal.REQUEST).show()  # noqa
        self.assertEqual(
            messages[-1].message,
            u'No reminder event fired')

        # schedule a must read and test again
        deadline = datetime.utcnow() + timedelta(1)
        self.tracker.schedule_must_read(self.page1, ['user1'], deadline)
        view = self.portal.restrictedTraverse('@@send-read-reminders')
        view()
        self.assertEqual(len(self.event_catcher.fired), 1)
        event = self.event_catcher.fired[0]
        self.assertTrue(
            IReadReminder.providedBy(event))
        self.assertEqual(event.object, self.page1)
        # a status message indicates if reminders have been sent
        messages = IStatusMessage(self.portal.REQUEST).show()  # noqa
        self.assertEqual(
            messages[-1].message,
            u'Reminder event fired for 1 objects: /plone/folder/page-1')

        # the send-read-reminder view is only checking objects within
        # it's context
        # calling it on folder-2 does not result in an events fired
        self.event_catcher.reset()
        view = self.folder2.restrictedTraverse('@@send-read-reminders')
        view()
        self.assertEqual(len(self.event_catcher.fired), 0)

    def test_send_expired_notification(self):
        # ordinary members must not access our view
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse,
                          '@@send-expired-notification')

        # site admins and managers have the proper permission
        setRoles(self.portal, TEST_USER_ID, ['Site Administrator'])
        # if we provide a site mail address this won't fail anymore
        sm = zope.component.getSiteManager(context=self.portal)
        sm.manage_changeProperties({'email_from_address': 'admin@site.com',
                                    'email_from_name': 'Website Adminstratör'})
        # calling our view w/o open requests results in no events fired
        view = self.portal.restrictedTraverse('@@send-expired-notification')
        view()

        # no expired read-requests, no mails sent out.
        messages = self.portal.MailHost.messages
        self.assertEqual(len(messages), 0)

        # create some expired read requests
        deadline = datetime.utcnow() - timedelta(hours=1)
        long_ago = datetime(2017, 1, 1, 14, 00)
        self.tracker.schedule_must_read(self.page1, ['user1'], deadline)
        self.tracker.schedule_must_read(self.file1, ['user1'], long_ago)
        self.tracker.mark_read(self.file1, 'user1', long_ago + timedelta(1))
        view()
        messages = self.portal.MailHost.messages
        self.assertEqual(len(messages), 1)
        msg = message_from_string(messages[0])
        self.assertEqual(msg['To'], u'admin@site.com')
        self.assertEqual(msg['From'], u'admin@site.com')
        self.assertEqual(msg['Subject'], '=?utf-8?q?Expired_read_requests?=')
        text = msg.get_payload()
        self.assertTrue((
            'The following items have open read requests with expired '
            'deadlines') in text)
        # page1 is listed, as the request has not been marked as read
        self.assertTrue('Page 1 (http://nohost/plone/folder/page-1)' in text)
        # file1 is listed too, as the request has been marked as read too late
        self.assertTrue(
            'File 1 (http://nohost/plone/folder-2/file1.txt)' in text)
        # user1 is listed for both objects
        self.assertEqual(text.count('user1@plone.org'), 2)

        # if we set the expired_recipient in the registry settings,
        # reports are sent to this address
        api.portal.set_registry_record('expired_recipient',
                                       [u'john@doe.com', u'foo@bar.baz'],
                                       IMustReadSettings)
        view()
        msg = message_from_string(messages[-1])
        self.assertEqual(msg['To'], 'john@doe.com, foo@bar.baz')

    def test_stats_csv(self):
        # ordinary members must not access our view
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse,
                          '@@read-statistic-csv')

        # site admins and managers have the proper permission
        setRoles(self.portal, TEST_USER_ID, ['Site Administrator'])
        view = self.portal.restrictedTraverse('@@read-statistic-csv')

        # no db entries, only headers are returned
        result = view()
        f = StringIO(result)
        reader = csv.DictReader(f)
        self.assertEqual(
            reader.fieldnames,
            ['path', 'userid', 'read_at', 'deadline', 'scheduled_at',
             'scheduled_by', 'status', 'uid', 'type'])
        row_count = sum(1 for _ in reader)
        self.assertEqual(row_count, 0)

        # create a db entry and call our view again
        self.tracker.mark_read(self.page1, 'user1',
                               datetime(2017, 4, 12, 13, 0))
        result = view()
        f = StringIO(result)
        reader = csv.DictReader(f)
        lines = [line for line in reader]
        self.assertEqual(len(lines), 1)
        self.assertDictContainsSubset(dict(
            status='read',
            read_at='2017-04-12 13:00:00',
            userid='user1',
            path='/plone/folder/page-1'),
            lines[0])

        # the view is context, dependent, and returns only db entries for
        # objects within the current context
        view = self.folder2.restrictedTraverse('@@read-statistic-csv')
        result = view()
        f = StringIO(result)
        reader = csv.DictReader(f)
        lines = [line for line in reader]
        self.assertEqual(len(lines), 0)



# -*- coding: utf-8 -*-
from plone.stringinterp.adapters import BaseSubstitution
from Products.CMFCore.interfaces import IContentish
from zope.component import adapts


class Fullname(BaseSubstitution):
    adapts(IContentish)

    category = u'Mustread'
    description = (
        u'Full Name of the person the read request is assigned to '
        u'(only available in `Request read confirmation` action)')

    def safe_call(self):
        return self.wrapper.mustread_fullname


class Deadline(BaseSubstitution):
    adapts(IContentish)

    category = u'Mustread'
    description = (
        u'Localized deadline of the read request '
        u'(only available in `Request read confirmation` action)')

    def safe_call(self):
        return self.wrapper.mustread_deadline


class URL(BaseSubstitution):
    adapts(IContentish)

    category = u'Mustread'
    description = u'URL of the item that marks the object as read'

    def safe_call(self):
        return self.context.absolute_url() + '/@@mark-read'



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import collective.contentrules.mustread


class CollectiveContentrulesMustreadLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=collective.contentrules.mustread)
        self.loadZCML(package=collective.contentrules.mustread,
                      name='archetypes.zcml')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.contentrules.mustread:default')


COLLECTIVE_CONTENTRULES_MUSTREAD_FIXTURE = CollectiveContentrulesMustreadLayer()  # noqa


COLLECTIVE_CONTENTRULES_MUSTREAD_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_CONTENTRULES_MUSTREAD_FIXTURE,),
    name='CollectiveContentrulesMustreadLayer:IntegrationTesting'
)


COLLECTIVE_CONTENTRULES_MUSTREAD_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_CONTENTRULES_MUSTREAD_FIXTURE,),
    name='CollectiveContentrulesMustreadLayer:FunctionalTesting'
)


COLLECTIVE_CONTENTRULES_MUSTREAD_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_CONTENTRULES_MUSTREAD_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='CollectiveContentrulesMustreadLayer:AcceptanceTesting'
)



# -*- coding: utf-8 -*-
from collective.contentrules.mustread.interfaces import IReadConfirmationRequest  # noqa
from collective.contentrules.mustread.interfaces import IReadReminder
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class ReadConfirmationRequestEvent(ObjectEvent):

    implements(IReadConfirmationRequest)


class ReadReminderEvent(ObjectEvent):

    implements(IReadReminder)



# -*- coding: utf-8 -*-

# default values for our content rule
# can be monkey patched by integrators

DEFAULT_DEADLINE_DELAY = 5
DEFAULT_REMINDER_DELAY = 2
DEFAULT_ACQUIRED_ROLES = True

DEFAULT_NOTIFICATION_SUBJECT = u'Read request'

DEFAULT_NOTIFICATION_TEXT = (
    u'Dear ${mustread_fullname}\n'
    u'${user_fullname} has requested you to read "${title}" before '
    u'${mustread_deadline} \n\n'
    u'${mustread_url}\n\n'
    u'Thanks in advance!')


DEFAULT_REMINDER_SUBJECT = u'Reminder: read request'

DEFAULT_REMINDER_TEXT = (
    u'Dear ${mustread_fullname}, please make sure to read "${title}" before '
    u'${mustread_deadline} \n\n'
    u'${mustread_url}')



# -*- coding: utf-8 -*-
from collective.mustread.interfaces import ITracker
from cStringIO import StringIO
from datetime import date
from Products.Five.browser import BrowserView
from zope.component import getUtility


class CSVExport(BrowserView):

    # allows to customize exported columns in subclasses
    fieldnames = []

    @property
    def filename(self):
        return 'must-read-{date:%Y-%m-%d}-{path}.csv'.format(
            date=date.today(),
            path='-'.join(self.context.getPhysicalPath()))

    def __call__(self):

        tracker = getUtility(ITracker)

        csv = StringIO()
        tracker.get_report_csv(csv, self.context, fieldnames=self.fieldnames)
        self.request.response.setHeader('Content-Type', 'text/csv')
        self.request.response.setHeader(
            'Content-Disposition', 'attachment; filename="{0}"'.format(
                self.filename))

        return csv.getvalue()



# -*- coding: utf-8 -*-
from collective.contentrules.mustread import _
from collective.contentrules.mustread import config
from collective.contentrules.mustread.interfaces import IReadReminder
from collective.mustread.interfaces import ITracker
from datetime import datetime
from datetime import timedelta
from email.utils import formataddr
from OFS.SimpleItem import SimpleItem
from plone import api
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from plone.stringinterp.interfaces import IContextWrapper
from plone.stringinterp.interfaces import IStringInterpolator
from zope import schema
from zope.component import adapts
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.formlib import form
from zope.i18nmessageid import MessageFactory
from zope.interface import implements
from zope.interface import Interface

import logging


logger = logging.getLogger('collective.contentrules.mustread')

_p = MessageFactory('plone')


class IMustReadAction(Interface):
    """Definition of the configuration available for a mustread action
    """

    source = schema.TextLine(
        title=_p(u'Email source'),
        description=_p(
            ('The email address that sends the email. If no email is '
             'provided here, it will use the portal from address.')),
        required=False)

    role = schema.Choice(
        title=_(u'field_role_title', default=u"Role"),
        description=_(
            u'field_role_description',
            default=(
                u"Select a role. The action will look up the all Plone site "
                u"users who explicitly have this role (also via groups) on the"
                u" object and send a message to their email address.")),
        vocabulary='collective.contentrules.mailtorole.roles',
        required=True)

    acquired = schema.Bool(
        title=_(u'field_acquired_title', default=u"Acquired Roles"),
        description=_(
            u'field_acquired_description',
            default=(
                u"Should users that have this role as an acquired role "
                u"also receive this email?")),
        required=False,
        default=config.DEFAULT_ACQUIRED_ROLES)

    global_roles = schema.Bool(
        title=_(u'field_global_roles_title', default=u"Global Roles"),
        description=_(
            u'field_global_roles_description',
            default=(
                u"Should users that have this role as a role in the "
                u"whole site also receive this email?")),
        required=False)

    notification_subject = schema.TextLine(
        title=_(u"Notification Subject"),
        description=_p(u"Subject of the message"),
        required=True,
        default=config.DEFAULT_NOTIFICATION_SUBJECT)

    notification_message = schema.Text(
        title=_(u"Notification Message"),
        description=_(
            u'field_message_description',
            default=(
                u"Type in here the message that you want to mail. Some "
                u"defined content can be replaced: ${title} will be replaced "
                u"by the title of the newly created item. ${url} will be "
                u"replaced by the URL of the newly created item.")),
        required=True,
        default=config.DEFAULT_NOTIFICATION_TEXT)

    reminder_subject = schema.TextLine(
        title=_(u"Reminder Subject"),
        description=_p(u"Subject of the message"),
        required=True,
        default=config.DEFAULT_REMINDER_SUBJECT)

    reminder_message = schema.Text(
        title=_(u"Reminder Message"),
        description=_(
            u'field_message_reminder_description',
            default=(
                u""
                u"leave empty to not send a reminder message")),
        required=False,
        default=config.DEFAULT_REMINDER_TEXT)

    deadline = schema.Int(
        title=_(u'label_deadline', default=u"Deadline Delay"),
        description=_(
            u'help_deadline',
            default=(
                u"Number of days a read requests must be confirmed within. "
                u"Will be used to compute the deadline of requests.")),
        required=True,
        default=config.DEFAULT_DEADLINE_DELAY
        )

    reminder_delay = schema.Int(
        title=_(u'label_reminder-delay', default=u"Reminder Delay"),
        description=_(
            u'help_reminder-delay',
            default=(
                u"Defines how many days before the deadline a "
                u"reminder email will be sent")),
        required=True,
        default=config.DEFAULT_REMINDER_DELAY
        )


class MustReadAction(SimpleItem):
    """
    The implementation of the action
    """
    implements(IMustReadAction, IRuleElementData)

    notification_subject = u''
    reminder_subject = u''
    source = u''
    role = u''
    notification_message = u''
    reminder_message = u''
    acquired = config.DEFAULT_ACQUIRED_ROLES
    global_roles = False
    deadline = config.DEFAULT_DEADLINE_DELAY
    reminder_delay = config.DEFAULT_REMINDER_DELAY
    element = 'plone.actions.MustRead'

    @property
    def summary(self):

        if self.reminder_message is None or (
                not self.reminder_message.strip()):
            return _(
                u'rule_description_noreminder',
                default=(
                    u"Request confirmation for users with role '${role}' "
                    u"on the object (deadline today + ${days} days, "
                    u"no reminder)"),
                mapping=dict(role=self.role,
                             days=self.deadline))
        return _(
            u'rule_description',
            default=(
                u"Request confirmation for users with role '${role}' on "
                u"the object (deadline today + ${days} days, reminder "
                u"${reminder} days before deadline)"),
            mapping=dict(role=self.role,
                         days=self.deadline,
                         reminder=self.reminder_delay))


class BaseExecutor(object):
    """Base Class for MustRead Executors"""

    @property
    def affected_users(self):
        """return users that match the contentrule's role filters"""
        obj = self.event.object
        recipients = set()

        # search through all local roles on the object, and add
        # userid to the recipients list if they have the local
        # role stored in the action
        local_roles = obj.get_local_roles()

        for user, roles in local_roles:
            rolelist = list(roles)
            if self.element.role in rolelist:
                recipients.add(user)

        # check for the acquired roles
        if self.element.acquired:
            sharing_page = obj.unrestrictedTraverse('@@sharing')
            acquired_roles = sharing_page._inherited_roles()
            try:
                acquired_roles += sharing_page._borg_localroles()
            except AttributeError:
                # ignore if method is missing
                pass
            acquired_users = [r[0] for r in acquired_roles
                              if self.element.role in r[1]]
            recipients.update(acquired_users)

        # check for the global roles
        if self.element.global_roles:
            pas = api.portal.get_tool('acl_users')
            rolemanager = pas.portal_role_manager
            global_role_ids = [
                p[0] for p in
                rolemanager.listAssignedPrincipals(self.element.role)]
            recipients.update(global_role_ids)

        # check to see if the recipients are users or groups
        group_ids = set()  # ids of affected groups - will be filtered out

        def _getGroupMemberIds(group):
            """Helper method to support groups in groups."""
            members = []
            for member_id in group.getGroupMemberIds():
                subgroup = api.group.get(member_id)
                if subgroup is not None:
                    members.extend(_getGroupMemberIds(subgroup))
                else:
                    members.append(member_id)
            return members

        for recipient in list(recipients):
            group = api.group.get(recipient)
            if group is not None:
                group_ids.add(recipient)
                recipients.update(_getGroupMemberIds(group))

        # remove group ids from recipients
        recipients.difference_update(group_ids)
        return sorted(recipients)

    def _send_mail(self, users, subject, text):
        mailhost = api.portal.get_tool('MailHost')
        portal = api.portal.get()

        # obtain email settings
        email_charset = portal.getProperty('email_charset')
        source = self.element.source
        if not source:
            # no source provided, looking for the site wide from email
            # address
            from_address = portal.getProperty('email_from_address')
            if not from_address:
                raise ValueError((
                    'You must provide a source address for this action or '
                    'enter an email in the portal properties'))
            from_name = portal.getProperty('email_from_name').strip('"')
            source = formataddr((from_name, from_address))

        # look up e-mail addresses for the found users
        for user in users:
            member = api.user.get(user)
            # check whether user really exists
            # before getting its email address
            if not member:
                logger.warn('non-existent user: ' + user)
                continue

            email = member.getProperty('email')
            if email is None or len(email) == 0:
                logger.warn('user without email address: ' + user)
                continue

            fullname = member.getProperty('fullname', '')
            recipient = formataddr((fullname, email))

            interpolator = self._get_interpolator(user)
            # Prepend interpolated message with \n to avoid interpretation
            # of first line as header.
            message = '\n' + interpolator(text)
            subject = interpolator(subject)

            mailhost.send(
                messageText=message,
                mto=recipient,
                mfrom=source,
                subject=subject,
                charset=email_charset)

    def _get_interpolator(self, userid):
        """adds a plone.stringinterp IContextWrapper
        around our event.object to add support for
        mustread_ variable substitution
        """
        obj = self.event.object
        tracker = getUtility(ITracker)
        deadlines = tracker.who_did_not_read(obj)
        deadline = deadlines.get(userid, None)
        if deadline:
            deadline = api.portal.get_localized_time(deadline, False)
        else:
            deadline = ''
        user = api.user.get(userid=userid)
        wrapped = IContextWrapper(obj)(
            mustread_deadline=deadline,
            mustread_fullname=user.getProperty('fullname'))
        return IStringInterpolator(wrapped)


class MustReadSendConfirmationExecutor(BaseExecutor):
    """schedules a read request for all users matching
    our role-filter on the event.object
    and notifies them via email

    registerd for iobjectevent so other contentrule events
    (i.e objectadded) can trigger our executor too.
    """
    implements(IExecutable)
    adapts(Interface, IMustReadAction, IObjectEvent)

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = self.event.object
        now = datetime.utcnow()
        deadline = now + timedelta(days=self.element.deadline)

        users = self.affected_users

        tracker = getUtility(ITracker)
        current_user = api.user.get_current().id
        new_users = tracker.schedule_must_read(obj, users, deadline,
                                               current_user)

        logger.info((
            'scheduled mustread on {path}, deadline '
            '{deadline:%Y-%m-%d %H:%M:%S} '
            'for {usercount} users: {users}').format(
                path='/'.join(obj.getPhysicalPath()),
                deadline=deadline,
                usercount=len(new_users),
                users=', '.join(new_users))
        )

        self._send_mail(new_users,
                        self.element.notification_subject,
                        self.element.notification_message)

        api.portal.show_message(_(
            u'msg-confirmation-requested',
            default=(
                u'Read confirmation requested by ${deadline} for ${count} '
                'users: ${userlist}'),
            mapping=dict(
                count=len(new_users),
                deadline=api.portal.get_localized_time(deadline, True),
                userlist=', '.join(new_users))),
            self.context.REQUEST,
            'info')


class MustReadReminderExecutor(BaseExecutor):
    """reminds users that have open read requests via email

    This can only be triggered by an IReadReminder event.
    (which is typically notified by the @@send-read-reminders view.

    .. ATTENTION::
        This reminderexecutor only compares the date of the deadline
        so it should only be called once a day to not send out multiple
        reminders for the same object.
        We might register different executors or add a parameter to
        the event to realize different behaviours at a later stage.
    """
    implements(IExecutable)
    adapts(Interface, IMustReadAction, IReadReminder)

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = self.event.object
        path = '/'.join(obj.getPhysicalPath())

        msg = self.element.reminder_message
        if not (msg and msg.strip()):
            logger.info((
                'no reminder message set, '
                'not sending a reminder to for {0}').format(path))
            return

        tracker = getUtility(ITracker)
        today = datetime.utcnow().date()
        remind_today = []
        too_early = []
        for userid, deadline in tracker.who_did_not_read(obj).iteritems():
            reminder_date = (
                deadline - timedelta(self.element.reminder_delay)).date()
            if reminder_date == today:
                remind_today.append(userid)
            if reminder_date > today:
                too_early.append((userid, deadline))

        if remind_today:
            logger.info(
                'sending {count} reminders for {path} to: {userlist}'.format(
                    count=len(remind_today), path=path,
                    userlist=', '.join(remind_today)
                ))
        if too_early:
            logger.info((
                u'too early to remind {count} users for {path}: '
                u'{userlist}').format(
                    count=len(too_early), path=path,
                    userlist=', '.join([
                        '{0}({1:%Y-%m-%d})'.format(*u) for u in too_early])
                    ))

        self._send_mail(remind_today, self.element.reminder_subject,
                        self.element.reminder_message)


form_description = _(
    u'form_description',
    default=(
        u"An action action that requests a read-confirmation from users who "
        u"have a role on the object")
    )


class MustReadAddForm(AddForm):
    """
    An add form for the mail action
    """
    form_fields = form.FormFields(IMustReadAction)
    label = _(u'title_addform', default=u"Add MustRead Action")
    description = form_description
    form_name = _p(u"Configure element")

    def create(self, data):
        a = MustReadAction()
        form.applyChanges(a, self.form_fields, data)
        return a


class MustReadEditForm(EditForm):
    """
    An edit form for the mail action
    """
    form_fields = form.FormFields(IMustReadAction)
    label = _(u'title_editform', default=u"Edit MustRead Action")
    description = form_description
    form_name = _p(u"Configure element")



# -*- coding: UTF-8 -*-
from collective.contentrules.mustread import _
from collective.contentrules.mustread.event import ReadConfirmationRequestEvent
from collective.contentrules.mustread.event import ReadReminderEvent
from collective.contentrules.mustread.interfaces import ICanBeMarkedAsMustRead
from collective.contentrules.mustread.interfaces import IMustReadSettings
from collective.mustread.interfaces import ITracker
from plone import api
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import event
from zope.component import getUtility
from zope.i18n import translate
from zope.i18nmessageid.message import Message

import datetime
import logging


logger = logging.getLogger('collective.contentrules.mustread')


class MustReadEnabled(BrowserView):
    """returns true if mustread actions
    (such as request read confirmations, mark as read)
    are  available on the context
    """

    def __call__(self):
        return ICanBeMarkedAsMustRead.providedBy(self.context)


class BaseView(BrowserView):

    def do_redirect(self):
        props = api.portal.get_tool('portal_properties').site_properties
        view_types = props.getProperty('typesUseViewActionInListings', [])
        url = self.context.absolute_url()
        if self.context.portal_type in view_types:
            url = url + '/view'
        self.request.response.redirect(url)


class RequestReadConfirmation(BaseView):
    """triggers a ReadConfirmationRequestEvent to notify
    contentrules engine
    """

    def __call__(self):
        event.notify(ReadConfirmationRequestEvent(self.context))
        self.do_redirect()


class MarkRead(BaseView):
    """marks context as read for currently logged in user
    """

    def __call__(self):
        tracker = getUtility(ITracker)
        tracker.mark_read(self.context)
        self.do_redirect()


class SendReminders(BaseView):
    """finds objects with unconfirmed read requests
    and notifies contentrules engine for each of them
    """

    def __call__(self):
        tracker = getUtility(ITracker)
        notified = []
        for obj in tracker.what_to_read(context=self.context):
            event.notify(ReadReminderEvent(obj))
            notified.append('/'.join(obj.getPhysicalPath()))

        if notified:
            api.portal.show_message(_(
                u'msg_reminder_fired',
                default=(
                    u'Reminder event fired for ${count} objects: ${objects}'),
                mapping={'count': len(notified),
                         'objects': ', '.join(notified)}),
                self.request,
                'info')
        else:
            api.portal.show_message(_(
                u'msg_no_reminder_fired',
                default=u'No reminder event fired'),
                self.request,
                'info')

        self.do_redirect()


class ExpiredNotificationEmail(BrowserView):
    """remind the portal admin when people have open read requests.
    """
    template = ViewPageTemplateFile('expired_email.pt')

    SUBJECT = _(u'expired-mail-subject',
                default=u'Expired read requests')

    def __call__(self):
        path = '/'.join(self.context.getPhysicalPath())
        logger.warn('processing expired notifications for context ' + path)
        tracker = getUtility(ITracker)
        items = tracker.what_to_read(context=self.context,
                                     force_deadline=True)
        data = []
        today_12_pm = datetime.datetime.combine(datetime.datetime.utcnow(),
                                                datetime.time.max)
        for item in items:
            path = '/'.join(item.getPhysicalPath())
            users = tracker.who_did_not_read(
                item, force_deadline=True,
                deadline_before=today_12_pm)

            if not users:
                # skip this item if there are no expired deadlines
                continue
            info = dict(item=item, users=[])
            for userid, deadline in users.iteritems():
                user = api.user.get(userid)
                info['users'].append(dict(
                    name=user.getProperty('fullname', u''),
                    email=user.getProperty('email'),
                    deadline=api.portal.get_localized_time(deadline, True)))
            data.append(info)
        if len(data) == 0:
            # nothing to do
            logger.info((u'no open read requests ending before '
                         '{0:%Y-%m-%d %h:%M} for {1}').format(today_12_pm,
                                                              path))
            return u'no open requests, no report sent'

        mail_text = self.template(
            self.context,
            self.request,
            data=data)
        portal = api.portal.get()
        email_charset = portal.getProperty('email_charset')
        mailhost = api.portal.get_tool('MailHost')
        from_address = portal.getProperty('email_from_address')
        recipient = api.portal.get_registry_record('expired_recipient',
                                                   IMustReadSettings,
                                                   [])
        if not recipient:
            recipient = from_address

        subject = self.SUBJECT
        if isinstance(subject, Message):
            subject = translate(subject, context=self.request)

        logger.info(u'Sending expired notification to {0}:\n{1}\n'.format(
            safe_unicode(recipient), safe_unicode(mail_text)))
        mailhost.send(
            messageText=mail_text,
            mto=recipient,
            mfrom=from_address,
            subject=subject,
            charset=email_charset)

        msg = _(u'The following report has been sent to ${email}:',
                mapping=dict(email=recipient))
        return u'{0}:\n\n:{1}'.format(translate(msg), mail_text)



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import safe_unicode
from zope.schema.vocabulary import SimpleVocabulary


def RolesVocabularyFactory(context):
    """Vocabulary Factory for Roles in the Plone Site"""

    # context is the content rule object which is stored in the plone site
    sharing_page = context.restrictedTraverse('@@sharing')
    roles = sharing_page.roles()
    sane_roles = [(safe_unicode(role['id']).encode('ascii', 'replace'),
                   safe_unicode(role['id'])) for role in roles]
    sane_roles.append(('Owner', 'Owner'))
    return SimpleVocabulary.fromItems(sane_roles)
